package awskit-s3

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Object.ChecksumSource

Sourcemodule Algorithm : sig ... end
Sourcemodule Type : sig ... end
Sourcemodule Mode : sig ... end
Sourcetype value = {
  1. algorithm : Algorithm.t;
  2. value : string;
}

Explicit checksum value supplied by the caller or returned for a part. The value is the base64/string payload expected by the selected algorithm.

Sourcetype response = {
  1. values : value list;
  2. checksum_type : Type.t option;
}

Modeled checksum headers returned by object and multipart operations.

Sourcetype summary = {
  1. algorithms : Algorithm.t list;
  2. checksum_type : Type.t option;
}

Compact checksum metadata returned by list operations.